﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.IO;
using System.Windows.Forms;
using System.Collections.ObjectModel;


namespace SphericalMercatorGridTest
{
    class GridFunctions
    {

        public static Dictionary<PointShape, double> GetBathyPoints(string textFilename)
        {
            StreamReader input = new StreamReader(textFilename);
            string[] values = new string[3];
            string line = string.Empty;
            PointShape pointShape = new PointShape();
            PointShape nextPoint = new PointShape();

            Dictionary<PointShape, double> bathyPoints = new Dictionary<PointShape, double>();
            MultipointShape bathyPointShape = new MultipointShape();
            List<double> bathyDepths = new List<double>();

            while (input.EndOfStream == false)
            {
                line = input.ReadLine();
                char isnumber = char.Parse(line.Substring(0, 1));
                if (char.IsNumber(isnumber) == true || isnumber.ToString() == "-")
                {
                    values = line.Split(new Char[] { ' ', ',', '\t' });
                    pointShape.X = Convert.ToDouble(values[0]);
                    pointShape.Y = Convert.ToDouble(values[1]);
                    bathyPointShape.Points.Add(new PointShape(pointShape.X, pointShape.Y));
                    double depth = Math.Abs(double.Parse(values[2]));
                    bathyDepths.Add(depth);
                }
            }
            input.Close();
            int i = 0;
            foreach (PointShape point in bathyPointShape.Points)
            {
                bathyPoints.Add(point, bathyDepths[i]);
                i++;
            }
            return bathyPoints;
        }

        public static GridFeatureLayer MakeGenericBathyGrid(WinformsMap winformsMap, string gridFilename, double areaBuffer, Dictionary<PointShape, double> bathyPoints)
        {
            double minX = bathyPoints.Keys.Min(p => p.X) - areaBuffer;
            double minY = bathyPoints.Keys.Min(p => p.Y) - areaBuffer;
            double maxX = bathyPoints.Keys.Max(p => p.X) + areaBuffer;
            double maxY = bathyPoints.Keys.Max(p => p.Y) + areaBuffer;

            double minDepth = bathyPoints.Values.Min();
            double maxDepth = bathyPoints.Values.Max();

            RectangleShape currentDrawingExtent = new RectangleShape(minX, maxY, maxX, minY);
            currentDrawingExtent.ScaleUp(20);
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, 1000, -9999, bathyPoints);

            Proj4Projection UTMToMercatorProjection = GetUTMToMercatorProjection();
            UTMToMercatorProjection.Open();
            winformsMap.CurrentExtent = UTMToMercatorProjection.ConvertToExternalProjection(currentDrawingExtent).GetBoundingBox();
            UTMToMercatorProjection.Close();

            FileStream gridFileStream = null;
            try
            {
                gridFileStream = new FileStream(gridFilename, FileMode.Create, FileAccess.ReadWrite);
                GridFeatureSource.GenerateGrid(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(2, areaBuffer), gridFileStream);
            }
            finally
            { if (gridFileStream != null) { gridFileStream.Close(); } }

            string layerName = new FileInfo(gridFilename).Name;
            GridFeatureLayer gridFeatureLayer = new GridFeatureLayer(gridFilename);

            GeoColor startColor = GeoColor.StandardColors.Red;
            GeoColor endColor = GeoColor.StandardColors.Blue;
            Collection<GeoColor> colors = GeoColor.GetColorsInQualityFamily(startColor, endColor, 10, ColorWheelDirection.Clockwise);

            if (maxDepth - minDepth == 0)
            {
                minDepth -= 50;
                maxDepth += 50;
            }

            Collection<double> isolineLevels = GenerateIsoLineLevels(Convert.ToDecimal(minDepth), Convert.ToDecimal(maxDepth), Convert.ToDecimal((maxDepth - minDepth) / 10));
            ClassBreakStyle classBreakLineStyle = new ClassBreakStyle(gridFeatureLayer.DataValueColumnName);

            int contoursCount = colors.Count;

            for (int j = 0; j < contoursCount; j++)
            {
                AreaStyle style = new AreaStyle(new GeoPen(GeoColor.FromArgb(50, colors[j]), 1), new GeoSolidBrush(GeoColor.FromArgb(50, colors[j])));
                classBreakLineStyle.ClassBreaks.Add(new ClassBreak(isolineLevels[j], style));
            }

            gridFeatureLayer.FeatureSource.Projection = UTMToMercatorProjection;//////////////set sphericalMercator projetion
            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(classBreakLineStyle);
            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return gridFeatureLayer;
        }

        public static Collection<double> GenerateIsoLineLevels(decimal min, decimal max, decimal interval)
        {
            Decimal minDecimal = Decimal.Remainder(min, interval);
            min = min - minDecimal;
            Decimal maxDecimal = Decimal.Remainder(max, interval);
            max = max - maxDecimal;

            Collection<double> levels = new Collection<double>();
            for (decimal i = min; i <= max; i = i + interval)
            { levels.Add(Convert.ToDouble(i)); }

            return levels;
        }

        public static Proj4Projection GetUTMToMercatorProjection()
        {
            Proj4Projection resultProjection = new Proj4Projection();
            resultProjection.InternalProjectionParametersString = string.Format("+proj=utm +zone={0} +{1} +ellps=WGS84 +datum=WGS84 +units=m +no_defs", "35", "north");//UTM
            resultProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            return resultProjection;
        }
    }
}
